<?

 /**********************************************************************************************
 * Filename: modules/network.php
 *
 * This file is part of the System statistics JAS plugin.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The System statistics JAS plugin is free software under the terms of the
 * GNU General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the plugin or http://jas.junerules.com/plugins/system_statistics/
 **********************************************************************************************/


 require_once($_path["include"]."dt.php");
 require_once($_path["include"]."module.php");

 class Plugin_Module_system_statistics_network extends Module
 {
  function __construct(&$data)
  {
   global $_path;

   parent::__construct($data);

   $this->data->title="Network system statistics";

   $this->data->defaultFields("id");

   $this->data->addField("keynr", "nr", "all", "reload", 1, false, 0, "Device");
   $this->data->addReference("keynr", "system_statistics_keys", "nr", "*:1");

   $this->data->addField("rx", "nr", "all", "big", 0, false, 0, "RX received");
   $this->data->addField("tx", "nr", "all", "big", 0, false, 0, "TX transmitted");
   $this->data->addField("received", "nr", "sql;show", "big", 0, false, 0, "Bytes received");
   $this->data->addField("transmitted", "nr", "sql;show", "big", 0, false, 0, "Bytes transmitted");
   $this->data->addField("receivedpersecond", "float", "show", "", 0, false, 0, "Bytes/s received");
   $this->data->addField("transmittedpersecond", "float", "show", "", 0, false, 0, "Bytes/s transmitted");
   $this->data->addField("timestamp", "datetime", "all", "title", 0, false, "now", "Timestamp");
   $this->data->addField("timepast", "nr", "sql;show", "big", 0, false, 0, "Time past");

   $this->data->addField("userrights", "rights", "", "allusers", 0, false, 256);

   $this->data->addIndex("keynr", "keynr");

   $this->data->sortOrder("timestamp-");
  }

  function addPostSubmitErrorCheck(){return $this->editPostSubmitErrorCheck();}
  function editPostSubmitErrorCheck()
  {
   $d=new Data($this->data->module, "view", "all", Array("format" => $this->data->dataformat));
   for($c=0;$c<$this->data->vmax;$c++)
   {
    $d->clearValues();
    $d->setValue("keynr", $this->data->getValue("keynr", $c));
    $d->setValue("timestamp", $this->data->getValue("timestamp", $c));
    $d->setOperator("timestamp", "<");
    $d->setOperator("timestamp", "<");
    $d->setValue("show_total", 1);
    $d->setShowFields("timestamp", "rx", "tx");
    $d->sortOrder("nr-");
    $d->get();

    if($d->next())
    {
     $rx=$this->data->getValue("rx", $c);
     if($rx<$d->v["rx"]) //new smaller then previous one: counter reset
      $received=($rx+pow(2,32))-$d->v["rx"];
     else
      $received=$rx-$d->v["rx"];

     $tx=$this->data->getValue("tx", $c);
     if($tx<$d->v["tx"]) //new smaller then previous one: counter reset
      $transmitted=($tx+pow(2,32))-$d->v["tx"];
     else
      $transmitted=$tx-$d->v["tx"];

     $timestamp=DT::dateConvertS($this->data->getValue("timestamp", $c), $this->data->field["timestamp"]["format"][$this->data->dataformat]["datetimeformat"]);
     $timepast=$timestamp-DT::dateConvertS($d->v["timestamp"], $d->field["timestamp"]["format"][$d->dataformat]["datetimeformat"]);
    }
    else
    {
     $received=0;
     $transmitted=0;
     $timepast=0;
    }

    $this->data->setValue("received", $received, $c);
    $this->data->setValue("transmitted", $transmitted, $c);
    $this->data->setValue("timepast", $timepast, $c);
   }

   return true;
  }

  function viewPreGetErrorCheck()
  {
   if($this->data->check("use", "show", "receivedpersecond"))
   {
    if(!$this->data->check("use", "show", "received"))
     $this->data->field["received"]["use"].=";show";
    if(!$this->data->check("use", "show", "timepast"))
     $this->data->field["timepast"]["use"].=";show";
   }

   if($this->data->check("use", "show", "transmittedpersecond"))
   {
    if(!$this->data->check("use", "show", "transmitted"))
     $this->data->field["transmitted"]["use"].=";show";
    if(!$this->data->check("use", "show", "timepast"))
     $this->data->field["timepast"]["use"].=";show";
   }

   return true;
  }

  function viewPostGet()
  {
   $last=false;
   for($c=0;$c<$this->data->vmax;$c++)
   {
    if($timepast=$this->data->getValue("timepast", $c))
    {
     if(($received=$this->data->getValue("received", $c))!==false)
      $this->data->setValue("receivedpersecond", number_format($received/$timepast, 3, $this->data->format[$this->data->dataformat]["decimalseparator"], $this->data->format[$this->data->dataformat]["thousandsseparator"]), $c);

     if(($transmitted=$this->data->getValue("transmitted", $c))!==false)
      $this->data->setValue("transmittedpersecond", number_format($transmitted/$timepast, 3, $this->data->format[$this->data->dataformat]["decimalseparator"], $this->data->format[$this->data->dataformat]["thousandsseparator"]), $c);
    }
    else
    {
     $this->data->setValue("receivedpersecond", 0);
     $this->data->setValue("transmittedpersecond", 0);
    }
   }

   return true;
  }
 }
?>
