<?

 /**********************************************************************************************
 * Filename: ss.php
 *
 * This file is part of the System statistics JAS plugin.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The System statistics JAS plugin is free software under the terms of the
 * GNU General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the plugin or http://jas.junerules.com/plugins/system_statistics/
 **********************************************************************************************/


 require_once($_path["include"]."fs.php");
 require_once($_path["include"]."cli.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."errors.php");
 require_once($_path["include"]."plugin.php");

 class SystemStatistics
 {
  function __construct($configKey)
  {
   $this->cli=new CLI();

   $this->e=&$this->cli->e;
   $this->e->l->read("plugin_systemstatistics");

   $this->config=false;
   $this->configKey=$configKey;

   $this->keys=false;

   if($this->configKey)
   {
    $d=Data::cache("system_statistics_pluginConfigs", "view", "all");
    $d->setValue("key", $this->configKey);
    $d->get(1);

    if(!$d->next())
    {
     $this->e->add("Unable to gather config: \"".$configKey."\"", "error");
     return false;
    }

    $this->config=$d->v;
    $this->config["db"]="dashboard_cli://";

    if($d->v["server"]["method"]=="SNMP")
     $this->config["db"].=$d->v["server"]["username"].($d->v["server"]["password"]?":".$d->v["server"]["password"]:"")."@".$d->v["server"]["hostname"].":".$d->v["server"]["port"]."/snmp/".($d->v["server"]["passphrase"]?"?passphrase=".$d->v["server"]["passphrase"]:"");
    else if($d->v["server"]["method"]=="SSH")
     $this->config["db"].=$d->v["server"]["username"]."@".$d->v["server"]["hostname"].":".$d->v["server"]["port"]."/ssh/";

    unset($d);
   }
  }

  function gather()
  {
   if($this->config)
   {
    if($this->config["load"])
     if(!$this->load())
      return false;

    if($this->config["networkDevices"])
     if(!$this->network())
      return false;

    if($this->config["diskspaceDevices"])
     if(!$this->diskspace())
      return false;

    if($this->config["diskstatsDevices"])
     if(!$this->diskstats())
      return false;

    if($this->config["sensorsDevices"])
     if(!$this->sensors())
      return false;
   }

   return true;
  }

  function key($keypath, $unit=false, $info=false)
  {
   if(!$this->keys)
   {
    $this->keys=Data::cache("system_statistics_keys", "tree", "down");
    $this->keys->clearValues();
    $this->keys->setValue("keynr", 0);
    $this->keys->get();
   }

   $this->keys->reset();
   while($this->keys->next())
   {
    if($this->keys->v["keypath"]==$keypath)
    {
     $nr=$this->keys->v["nr"];

     if(($unit!==false&&$unit!=$this->keys->v["unit"])||($info!==false&&$info!=$this->keys->v["info"]))
     {
      $this->keys->action="edit";
      $this->keys->extra="item";
      $this->keys->clearValues();

      $this->keys->setValue("unit", $unit);
      $this->keys->setValue("info", $info);
      $this->keys->setValue("nr", $nr);
      $this->keys->submit();

      $this->keys->action="tree";
      $this->keys->extra="down";
      $this->keys->clearValues();
      $this->keys->setValue("keynr", 0);
      $this->keys->get();
     }

     return $nr;
    }
   }
 
   $path="";
   $lastkey=0;
   $this->keys->reset();

   $keypath=explode("->", $keypath);
   $l=sizeof($keypath);

   for($c=0;$c<$l;$c++)
   {
    if(!$path)
     $path=$keypath[$c];
    else 
     $path.="->".$keypath[$c];

    $found=false;

    while($this->keys->next())
    {
     if($this->keys->v["keypath"]==$path)
     {
      $lastkey=$this->keys->v["nr"];
      $found=true;
      break;
     }
    }

    if(!$found)
    {
     $this->keys->action="add";
     $this->keys->extra="item";
     $this->keys->clearValues();

     $this->keys->setDefaultValue("userrights");
     $this->keys->setValue("key", $keypath[$c]);
     $this->keys->setValue("keynr", $lastkey);

     if($c==($l-1)) //only safe units and info for last key
     {
      $this->keys->setValue("unit", $unit);
      $this->keys->setValue("info", $info);
     }
     else
     {
      $this->keys->setValue("unit", "");
      $this->keys->setValue("info", "");
     }

     $this->keys->submit();
     $lastkey=$this->keys->getValue("nr");
     
     $this->keys->action="tree";
     $this->keys->extra="down";
     $this->keys->clearValues();
     $this->keys->setValue("keynr", 0);
     $this->keys->get();
    }
   }

   return $lastkey;
  }


/*********************
LOAD 
*********************/

  function load()
  {
   if($this->e->any("fatal"))
    return false;

   $found=false;
   $d=Data::cache("system_statistics_load", "add", "item");
   $ds=Data::cache("dashboard_uptime", "view", "all", Array("db" => $this->config["db"], "format" => "sql"));
   $ds->setShowFields("users", "load1", "load5", "load15", "activeprocs", "totalprocs");

   if(!$ds->get())
   {
    $this->e->add($ds->e);
    return false;
   }

   while($ds->next())
   {
    $d->setValue("users", $ds->v["users"]); 
    $d->setValue("load1", $ds->v["load1"]); 
    $d->setValue("load5", $ds->v["load5"]); 
    $d->setValue("load15", $ds->v["load15"]); 
    $d->setValue("activeprocs", $ds->v["activeprocs"]); 
    $d->setValue("totalprocs", $ds->v["totalprocs"]); 
 
    $d->setValue("keynr", $this->key($this->config["key"]."->System load", "Procs", "Active processes")); 

    if(!$d->submit())
     $this->e->add($d->e);
   }

   if($this->e->any("fatal"))
    return false;

   return true;
  }


/*********************
NETWORK
*********************/

  function network()
  {
   if($this->e->any("fatal"))
    return false;

   $d=Data::cache("system_statistics_network", "add", "item");
   $ds=Data::cache("dashboard_network", "view", "all", Array("db" => $this->config["db"], "format" => "sql"));
   $ds->setShowFields("device", "rx", "tx");

   if(!$ds->get())
   {
    $this->e->add($ds->e);
    return false;
   }

   while($ds->next())
   {
    if(!in_array($ds->v["device"], $this->config["networkDevices"]))
     continue;

    $d->setValue("keynr", $this->key($this->config["key"]."->Networking->".$ds->v["device"], "bytes")); 

    $d->setValue("rx", $ds->v["rx"]); 
    $d->setValue("tx", $ds->v["tx"]); 

    if(!$d->submit())
     $this->e->add($d->e);
   }

   if($this->e->any("fatal"))
    return false;

   return true;
  }


/*********************
DISK SPACE 
*********************/

  function diskspace()
  {
   if($this->e->any("fatal"))
    return false;

   $d=Data::cache("system_statistics_diskspace", "add", "item");
   $ds=Data::cache("dashboard_diskspace", "view", "all", Array("db" => $this->config["db"], "format" => "sql"));
   $ds->setShowFields("mountpoint", "device", "available", "used", "total");

   if(!$ds->get())
   {
    $this->e->add($ds->e);
    return false;
   }

   while($ds->next())
   {
    if(!in_array($ds->v["mountpoint"], $this->config["diskspaceDevices"]))
     continue;

    $d->setValue("keynr", $this->key($this->config["key"]."->Disk space->".$ds->v["mountpoint"], "bytes", $ds->v["device"])); 

    $d->setValue("available", $ds->v["available"]); 
    $d->setValue("used", $ds->v["used"]); 
    $d->setValue("total", $ds->v["total"]); 

    if(!$d->submit())
     $this->e->add($d->e);
   }

   if($this->e->any("fatal"))
    return false;

   return true;
  }


/*********************
DISK STATISTICS
*********************/

  function diskstats()
  {
   if($this->e->any("fatal"))
    return false;

   $d=Data::cache("system_statistics_diskstats", "add", "item");
   $ds=Data::cache("dashboard_diskstats", "view", "all", Array("db" => $this->config["db"], "format" => "sql"));
   $ds->setShowFields("deviceattribute", "device", "family", "model", "attributenr", "attribute", "value", "unit");

   if(!$ds->get())
   {
    $this->e->add($ds->e);
    return false;
   }

   while($ds->next())
   {
    if(!in_array($ds->v["deviceattribute"], $this->config["diskstatsDevices"]))
     continue;

    $d->setValue("keynr", $this->key($this->config["key"]."->Disk stats->".$ds->v["device"]."->".$ds->v["attributenr"], $ds->v["unit"], $ds->v["family"]." - ".$ds->v["model"]." (".$ds->v["attribute"].")")); 
    $d->setValue("value", $ds->v["value"]); 

    if(!$d->submit())
     $this->e->add($d->e);
   }

   if($this->e->any("fatal"))
    return false;

   return true;
  }


/*********************
SENSORS
*********************/

  function sensors()
  {
   if($this->e->any("fatal"))
    return false;

   $d=Data::cache("system_statistics_sensors", "add", "item");
   $ds=Data::cache("dashboard_sensors", "view", "all", Array("db" => $this->config["db"], "format" => "sql"));
   $ds->setShowFields("devicesensor", "device", "adapter", "sensor", "unit", "value");

   if(!$ds->get())
   {
    $this->e->add($ds->e);
    return false;
   }

   while($ds->next())
   {
    if(!in_array($ds->v["devicesensor"], $this->config["sensorsDevices"]))
     continue;

    $d->setValue("keynr", $this->key($this->config["key"]."->Sensors->".$ds->v["device"]."->".$ds->v["sensor"], $ds->v["unit"], $ds->v["adapter"])); 
    $d->setValue("value", $ds->v["value"]); 

    if(!$d->submit())
     $this->e->add($d->e);
   }

   if($this->e->any("fatal"))
    return false;

   return true;
  }

 }
?>
