<?

 /**********************************************************************************************
 * Filename: system_statistics.php
 *
 * This file is part of the System statistics JAS plugin.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The System statistics JAS plugin is free software under the terms of the
 * GNU General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the plugin or http://jas.junerules.com/plugins/system_statistics/
 **********************************************************************************************/


 require_once($_path["include"]."dt.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."form.php");
 require_once($_path["include"]."functions.php");

 class Plugin_system_statistics extends Base_Plugin
 {
  function main()
  {
   if(!$this->config)
    return false;

   return true;
  }

  function output()
  {
   return $this->graph($this->config["key"]);
  }

  function graph($configKey)
  {
   global $_path;

   //graph line colors
   $colors=Array(
    "4488DD", "44DD88", "8844DD", "88DD44", "DD8844", "DD4488",
    "0066AA", "00AA66", "6600AA", "66AA00", "AA6600", "AA0066",
    "88DDFF", "88FFDD", "DD88FF", "DDFF88", "FFDD88", "FF88DD",
   );

   //get graphics plugin
   $graphplugin=new Plugin("graphics");

   if($graphplugin->e->any("fatal"))
   {
    $this->e->add($graphplugin->e);
    return false;
   }
   else if(!$graphplugin->isEnabled())
   {
    $this->e->add("Graphs plugin isn't enabled");
    return false;
   }

   require_once($graphplugin->path["root"]."graph.php");
   unset($graphplugin);


   //get config
   $d=Data::cache("system_statistics_pluginConfigs", "view", "all");
   $d->setValue("plugin", $this->plugin->name);
   $d->setValue("key", $configKey);

   if(!$d->get()||!$d->next())
   {
    $this->e->add($d->e);
    return false;
   }

   //base structure
   $items=Array(
    "load" => Array(
     "index" => "load",
     "key" => "System load",
     "title" => "",
     "keys" => Array(
      "users" => Array("title" => "", "status" => "unused", "selected" => false, "id" => 0, "unit" => "Users", "info" => ""),
      "load1" => Array("title" => "", "status" => "unused", "selected" => false, "id" => 0, "unit" => "Load", "info" => ""),
      "load5" => Array("title" => "", "status" => "unused", "selected" => false, "id" => 0, "unit" => "Load", "info" => ""),
      "load15" => Array("title" => "", "status" => "unused", "selected" => false, "id" => 0, "unit" => "Load", "info" => ""),
      "activeprocs" => Array("title" => "", "status" => "unused", "selected" => false, "id" => 0, "unit" => "Processes", "info" => ""),
      "totalprocs" => Array("title" => "", "status" => "unused", "selected" => false, "id" => 0, "unit" => "Processes", "info" => ""),
     ),
     "subkeys" => 0,
     "selected" => false,
     "fields" => Array(),
     "fieldmatchkey" => true,
    ),
    "network" => Array(
     "index" => "networkDevices",
     "key" => "Networking",
     "title" => "",
     "keys" => Array(),
     "subkeys" => 1,
     "selected" => false,
     "fields" => Array("transmittedpersecond", "receivedpersecond"),
     "fieldmatchkey" => false,
    ),
    "diskspace" => Array(
     "index" => "diskspaceDevices",
     "key" => "Disk space",
     "title" => "",
     "keys" => Array(),
     "subkeys" => 1,
     "selected" => false,
     "fields" => Array("used", "total"),
     "fieldmatchkey" => false,
    ),
    "diskstats" => Array(
     "index" => "diskstatsDevices",
     "key" => "Disk stats",
     "title" => "",
     "keys" => Array(),
     "subkeys" => 2,
     "selected" => false,
     "fields" => Array("value"),
     "fieldmatchkey" => false,
    ),
    "sensors" => Array(
     "index" => "sensorsDevices",
     "key" => "Sensors",
     "title" => "",
     "keys" => Array(),
     "subkeys" => 2,
     "selected" => false,
     "fields" => Array("value"),
     "fieldmatchkey" => false,
    ),
   );

   //setup titles
   foreach(array_keys($items) as $i)
    $items[$i]["title"]=$d->title($items[$i]["index"]);

   //setup load key titles
   $dl=Data::cache("system_statistics_load", "view", "all");
   foreach(array_keys($items["load"]["keys"]) as $k)
    $items["load"]["keys"][$k]["title"]=$dl->title($k);

   unset($dl);

   //setup other keys
   foreach(array_keys($items) as $i)
   {
    if($i=="load") //skip load
     continue;

    foreach($d->v[$items[$i]["index"]] as $k)
    {
     if($i=="diskstats")
      $k=str_replace(":", "->", $k); //fix attribute subkey (/dev/sda:190 -> /dev/sda->190)

     $items[$i]["keys"][$k]=Array("title" => $k, "status" => "unused", "selected" => false, "id" => 0, "unit" => "", "info" => "");
    }
   }

   //setup used/old keys
   $dk=Data::cache("system_statistics_keys", "view", "all");
   $dk->setShowFields("nr", "keypath", "unit", "info");
   $dk->sortOrder("keypath");

   if(!$dk->get())
   {
    $this->e->add($dk->e);
    return false;
   }

   foreach(array_keys($items) as $i)
   {
    $dk->reset();
    if($i=="load") //load only has 1 key for all fields
    {
     while($dk->next())
     {
      if(strpos($dk->v["keypath"], $d->v["key"]."->".$items[$i]["key"])===0) //used
      {
       foreach(array_keys($items["load"]["keys"]) as $k)
       {
        $items[$i]["keys"][$k]["status"]="used";
        $items[$i]["keys"][$k]["id"]=$dk->v["nr"];
       }
      }
     }
    }
    else
    {
     while($dk->next())
     {
      if(strpos($dk->v["keypath"], $d->v["key"]."->".$items[$i]["key"])===0) //used/old key
      {
       $k=str_replace($d->v["key"]."->".$items[$i]["key"], "", $dk->v["keypath"]);

       if($items[$i]["subkeys"]!=substr_count($k, "->")) //number of subkeys need to match
        continue;

       $k=preg_replace("/^->/", "", $k); //remove -> prefix

       if(!isset($items[$i]["keys"][$k]))
        $items[$i]["keys"][$k]=Array("title" => $k, "status" => "old", "selected" => false, "id" => 0, "unit" => "", "info" => "");
       else
        $items[$i]["keys"][$k]["status"]="used";

       $items[$i]["keys"][$k]["id"]=$dk->v["nr"];

       if(!$items[$i]["keys"][$k]["unit"])
        $items[$i]["keys"][$k]["unit"]=$dk->v["unit"];
       if(!$items[$i]["keys"][$k]["info"])
        $items[$i]["keys"][$k]["info"]=$dk->v["info"];
      }
     }
    }
   }

   unset($dk);


   //get from and to date time request vars
   if(!($from=request("from"))||!($from=DT::dateConvertS($from, $d->format["form"]["datetimeformat"])))
    $from=mktime(0, 0, 0);

   if(!($to=request("to"))||!($to=1+DT::dateConvertS($to, $d->format["form"]["datetimeformat"])))
    $to=86400+$from;

   if($from==$to)
    $to++;
   else if($from>$to)
   {
    $tmp=$from;
    $from=$to;
    $to=$tmp;
   }

   //aggregation
   if(($to-$from)>=(86400*365)) //period 1 year or more
   {
    $aggregationText="Monthly";
    $aggregation=86400*30.42; //aggregate per month
    $xDateFormat="M y"; //X axis date/time format: Jan 11
   }
   else if(($to-$from)>=(86400*90)) //period about 3 months or more
   {
    $aggregationText="Weekly";
    $aggregation=86400*7; //aggregate per day
    $xDateFormat="M y"; //X axis date/time format: Jan 11
   }
   else if(($to-$from)>=(86400*28)) //period 4 weeks or more
   {
    $aggregationText="Daily";
    $aggregation=86400; //aggregate per day
    $xDateFormat="j M"; //X axis date/time format: 2 Jan
   }
   else if(($to-$from)>=(86400*21)) //period 3 weeks or more
   {
    $aggregationText="4 Hourly";
    $aggregation=86400/6; //aggregate per 4 hours
    $xDateFormat="G:i j M"; //X axis date/time format: 12:00 2 Jan
   }
   else if(($to-$from)>=(86400*14)) //period 2 weeks or more
   {
    $aggregationText="2 Hourly";
    $aggregation=86400/12; //aggregate per 2 hours
    $xDateFormat="G:i D"; //X axis date/time format: 12:00 Mon
   }
   else if(($to-$from)>=(86400*7)) //period 1 week or more
   {
    $aggregationText="Hourly";
    $aggregation=3600; //aggregate per hour
    $xDateFormat="G:i D"; //X axis date/time format: 12:00 Mon
   }
   else if(($to-$from)>(86400*2)) //period more then 2 days
   {
    $aggregationText="Half hourly";
    $aggregation=1800; //aggregate per half hour
    $xDateFormat="G:i"; //X axis date/time format: 12:30
   }
   else //less then 1 week
   {
    $aggregationText="None";
    $aggregation=1; //don't aggregate
    $xDateFormat="G:i"; //X axis date/time format: 8:34
   }

   //get statistics types request vars
   $selected=false;
   foreach(array_keys($items) as $i)
   {
    if(!($tmp=request($items[$i]["index"]))||!is_array($tmp))
     continue;

    foreach($tmp as $k)
    {
     if(isset($items[$i]["keys"][$k]))
     {
      $selected=true; //at least one key selected
      $items[$i]["selected"]=true;
      $items[$i]["keys"][$k]["selected"]=true;

      if($i=="load") //add load fields
       $items[$i]["fields"][]=$k;
     }
    }
   }


   //render form
   $output="";

   $f=new Form();
   $output.=$f->start();

   foreach($this->plugin->vars as $k => $v)
    $output.=$f->input("hidden", $k, $v)."\n";

   $output.=$f->input("hidden", "graph", $d->getValue("key"))."\n";

   $output.="<div class=\"field\"><div class=\"title\">Date</div><div class=\"input\">\n";
   $output.=$f->input("text", "from", date($d->format["form"]["datetimeformat"], $from));
   $output.=" Till<br/>";
   $output.=$f->input("text", "to", date($d->format["form"]["datetimeformat"], $to-1));
   $output.="</div></div>\n";

   $output.="<div class=\"field\"><div class=\"title\">Aggregation</div><div class=\"input\">\n";
   $output.=$aggregationText;
   $output.="</div></div>\n";


   $output.="<div class=\"field\"><table class=\"overview\">";

   $output.="<tr>\n";

   foreach(array_keys($items) as $i)
    $output.="<th>".$items[$i]["title"]."</th>\n";

   $output.="</tr><tr>\n";

   foreach(array_keys($items) as $i)
   {
    $output.="<td><div>";

    foreach(array_keys($items[$i]["keys"]) as $k)
     $output.=$f->checkbox($items[$i]["keys"][$k]["selected"], $items[$i]["keys"][$k]["title"], $items[$i]["index"]."[]", $k, $items[$i]["keys"][$k]["status"], false, ($items[$i]["keys"][$k]["status"]=="unused"?"disabled=\"disabled\"":""))."<br/>";

    $output.="</div></td>\n";
   }

   $output.="</tr>\n</table></div>\n";


   $output.="<div class=\"field\"><div class=\"title\"></div><div class=\"input\">".$f->input("submit", "s", "Show")."</div></div>\n";
   $output.=$f->end();


   //nothing selected, done
   if(!$selected)
   {
    if(request("s"))
    {
     $this->e->add("No graph fields selected");
     $output.=$this->e->show();
    }
    return $output;
   }


   //configure graph
   $graph=new Graph();

   $graph->axis("x", "horizontal", $from, $to);
   $graph->dateFormat("x", $d->format["html"]["datetimeformat"]);
   $graph->interval("x", ($to-$from)/12);
   $graph->indicator("x");
   $graph->labels("x");
   $graph->color("x", "888888");

   $graph->grid("x");
   $graph->color("grid_x", "EEEEEE");

   $graph->text("x", "Time", 1);
   $graph->color("x_l1", false, false, "888888"); //axe title color


   $graph->axis("y1", "x", 0);
   $graph->display("y1", "humanreadable");

   $graph->indicator("y1");
   $graph->labels("y1");
   $graph->color("y1", "888888");

   $graph->grid("y1");
   $graph->color("grid_y1", "CCCCCC");


   $graph->axis("y2", "x", 0);
   $graph->display("y2", "humanreadable");

   $graph->indicator("y2");
   $graph->labels("y2");


   $graph->color("y2", "888888");


   //render graph
   $output.="<h2>".$d->v["key"]."</h2>";
   $output.=$graph->style();

   $baseField="timestamp";
   $legendY1=$d->v["height"]+20;
   $legendY2=$d->v["height"]+20;
   $averageY=false;
   foreach(array_keys($items) as $i) //run through all keys and accompanying modules
   {
    $dr=Data::cache($this->plugin->name."_".$i, "view", "all");

    foreach($items[$i]["keys"] as $k => $key)
    {
     if(!$key["selected"])
      continue;

     //get all values for the key within the from/to period
     $dr->clearValues();
     $dr->setValue("keynr", $key["id"], 0);
     $dr->setValue("keynr", $key["id"], 1);
     $dr->setValue($baseField, date($d->format["form"]["datetimeformat"], $from), 0);
     $dr->setValue($baseField, date($d->format["form"]["datetimeformat"], $to), 1);
     $dr->setOperator($baseField, ">=", 0);
     $dr->setOperator($baseField, "<", 1);

     //setup aggregation
     if($aggregation)
     {
      $dr->setValue("aggregation_field", $baseField); //use timestamp as aggregation_field
      $dr->setValue("aggregation_type", "average"); //show average value per period
      $dr->setValue("aggregation_method", "before"); //timestamp will indicate the coming period (4 hours, day, months, etc)
      $dr->setValue("aggregation_interval", $aggregation); //determined above based on from/to period
     }

     $dr->setShowFields(array_merge(Array($baseField), $items[$i]["fields"]));
     $dr->sortOrder($baseField);
     $dr->get();

     if($dr->vmax) //results?
     {
      $axe=false; //needed for fields grouping
      foreach($items[$i]["fields"] as $field) //render each field
      {
       if($items[$i]["fieldmatchkey"]&&$k!=$field) //skip fields that don't match key
        continue;

       if(!$color=array_shift($colors))
        $color="444444";

       //create values list for graph
       $tmp=Array();
       for($c=0;$c<$dr->vmax;$c++)
        $tmp[]=$dr->getValue($baseField, $c).",".$dr->sqlValue($field, $c);

       //determine which axe this set of fields should be rendered to
       if($axe==false||$i=="load")
       {
        //calculate average value for field
        $total=0;
        for($c=0;$c<$dr->vmax;$c++)
         $total+=$dr->sqlValue($field, $c);

        $average=$total/$dr->vmax;

        if($averageY===false) //Y axe doesn't have an avery set yet
         $averageY=$average; //use first

        if($average>($averageY*1.5)||$average<($averageY*.5)) //field average more then 50% lower or higher then Y axe average
         $axe="y2"; //use y2 (right)
        else
         $axe="y1"; //use y1 (left)
       }

       //set up line
       $graph->line($key["id"].$field, $axe, $tmp);
       $graph->width($key["id"].$field, 1);
       $graph->color($key["id"].$field, $color);


       //determine which legend block the legende text should be rendered to
       $width=($d->v["width"]/2);
       if($axe=="y1")
        $graph->rectangle("legend".$key["id"].$field, "0,".$legendY1, $width.",".$legendY1+=20);
       else
        $graph->rectangle("legend".$key["id"].$field, $width.",".$legendY2, ($width*2).",".$legendY2+=20);

       //render legend text
       if($i=="sensors")
        $graph->text("legend".$key["id"].$field, $key["title"]." (".$key["unit"].")");
       else if($i=="diskstats")
        $graph->text("legend".$key["id"].$field, $key["title"].": ".preg_replace("/^.+ - (.*) \(.*\)/", "\\1", $key["info"])." (".$key["unit"].")");
       else if($i=="network"||$i=="diskspace")
        $graph->text("legend".$key["id"].$field, $key["title"].": ".$dr->field[$field]["title"]);
       else
        $graph->text("legend".$key["id"].$field, $dr->field[$field]["title"]);

       $graph->textAlign("legend".$key["id"].$field, "left");
       $graph->color("legend".$key["id"].$field, false, false, $color);

       if($axe=="y2")
        $graph->textAlign("legend".$key["id"].$field, "right");
      }
     }
    }
   }

   if(!$graph->items["y1"]["rinterval"])
   {
    $graph->_determineAxe("y1");

    $max=$graph->items["y1"]["max"];
    $max=ceil($max); //remove decimals
    $length=(int)strlen((string)$max)-1; //get length (12345 -> 4)
    $max=ceil($max/pow(10, $length))*pow(10, $length); //round to nearest whole number *(12345 -> 20000)
    $graph->items["y1"]["rmax"]=$max; //set new max value

    $graph->interval("y1", ($max-$graph->items["y1"]["min"])/10); //set interval (10 steps)
   }

   if(isset($graph->items["y2"])&&!$graph->items["y2"]["rinterval"])
   {
    $graph->_determineAxe("y2");

    if($max=$graph->items["y2"]["max"])
    {
     $max=ceil($max); //remove decimals
     $length=(int)strlen((string)$max)-1; //get length (12345 -> 4)
     $max=ceil($max/pow(10, $length))*pow(10, $length); //round to nearest whole number *(12345 -> 20000)
     $graph->items["y2"]["rmax"]=$max; //set new max value

     $graph->interval("y2", ($max-$graph->items["y2"]["min"])/10); //set interval (10 steps)
    }
   }

   $graph->dateFormat("x", $xDateFormat);

   if($render=$graph->render($d->v["width"], $d->v["height"]))
    $output.=$render;
   else
    $output.=$graph->e->show();

   return $output;
  }
 }

?>
